#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#

#
# Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
#

import xdrlib
import threading
import rad.recordmarking as recordmarking
import rad.client

#
# Base adaptor classes, which do the bare minimum necessary.
# Exceptions generated by the rad client are passed straight through.
# Subclasses are free to map exceptions to meet their needs.
#

class RawAdaptorMethod(object):
    def __init__(self, realmeth):
        self._method = realmeth

    def __call__(self, *args):
	return self._method.call(*args)

class RawAdaptor(object):
    """ Makes a native-looking python object that throws RAD exceptions """

    def __init__(self, realobj, adapt = RawAdaptorMethod):
	# We complicate __init__ slightly to streamline __getattr__/__setattr__
	self.__dict__["_object"] = realobj
	self.__dict__.update(dict([[i._name, adapt(i)]
	    for i in realobj._methods.values()]))

    def __getattr__(self, attr):
    	obj = self.__dict__["_object"]
    	try:
		return obj.read(attr)
	except KeyError as e:
		raise AttributeError("%s has no attribute %s" %
		    (obj._name, attr))

    def __setattr__(self, attr, value):
    	obj = self.__dict__["_object"]
    	try:
		obj.write(attr, value)
	except KeyError as e:
		raise AttributeError("%s has no attribute %s" %
		    (obj._name, attr)) 

#
# Adaptor classes that map all errors to Python-like exceptions.
#

class AdaptorMethod(RawAdaptorMethod):
    def __call__(self, *args):
	try:
             return RawAdaptorMethod.__call__(self, *args)
	except Exception as e:
	     raise AttributeError(e)

class Adaptor(RawAdaptor):
    """ Makes a native-looking python object """

    def __init__(self, realobj):
    	RawAdaptor.__init__(self, realobj, AdaptorMethod)

    def __getattr__(self, attr):
    	try:
		return RawAdaptor.__getattr__(self, attr)
	except AttributeError: raise
	except rad.client.InvalidFeatureError as e:
		raise AttributeError(e.get_message())
	except Exception as e:
		raise AttributeError(e)

    def __setattr__(self, attr, value):
    	try:
		RawAdaptor.__setattr__(self, attr, value)
	except AttributeError: raise
	except rad.client.InvalidFeatureError as e:
		raise AttributeError(e.get_message())
	except Exception as e:
		raise AttributeError(e)
